@@ -1,7 +1,8 @@
 from unittest import TestCase
 from mock import patch, Mock
-from samtranslator.intrinsics.actions import Action, RefAction, SubAction, GetAttAction
+from samtranslator.intrinsics.actions import Action, RefAction, SubAction, GetAttAction, FindInMapAction
 from samtranslator.intrinsics.resource_refs import SupportedResourceReferences
+from samtranslator.model.exceptions import InvalidTemplateException, InvalidDocumentException
 
 class TestAction(TestCase):
 
@@ -419,6 +420,23 @@ class TestSubCanResolveParameterRefs(TestCase):
 
         self.assertEqual(expected, result)
 
+    def test_sub_all_refs_with_pseudo_parameters(self):
+        parameters = {
+            "key1": "value1",
+            "AWS::Region": "ap-southeast-1"
+        }
+        input = {
+            "Fn::Sub": "hello ${AWS::Region} ${key1}"
+        }
+        expected = {
+            "Fn::Sub": "hello ap-southeast-1 value1"
+        }
+
+        sub = SubAction()
+        result = sub.resolve_parameter_refs(input, parameters)
+
+        self.assertEqual(expected, result)
+
 class TestSubInternalMethods(TestCase):
 
     @patch.object(SubAction, "_sub_all_refs")
@@ -915,3 +933,201 @@ class TestGetAttCanResolveResourceIdRefs(TestCase):
         getatt = GetAttAction()
         can_handle_mock.return_value = False # Simulate failure to handle the input. Result should be same as input
         self.assertEqual(expected, getatt.resolve_resource_id_refs(input, self.supported_resource_id_refs))
+
+
+class TestFindInMapCanResolveParameterRefs(TestCase):
+
+    def setUp(self):
+        self.ref = FindInMapAction()
+
+    @patch.object(FindInMapAction, "can_handle")
+    def test_cannot_handle(self, can_handle_mock):
+        input = {
+            "Fn::FindInMap": ["a", "b", "c"]
+        }
+        can_handle_mock.return_value = False
+        output = self.ref.resolve_parameter_refs(input, {})
+
+        self.assertEqual(input, output)
+
+    def test_value_not_list(self):
+        input = {
+            "Fn::FindInMap": "a string"
+        }
+        with self.assertRaises(InvalidDocumentException):
+            self.ref.resolve_parameter_refs(input, {})
+
+    def test_value_not_list_of_length_three(self):
+        input = {
+            "Fn::FindInMap": ["a string"]
+        }
+
+        with self.assertRaises(InvalidDocumentException):
+            self.ref.resolve_parameter_refs(input, {})
+
+    def test_mapping_not_string(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": [["MapA"], "TopKey2", "SecondKey1"]
+        }
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(input, output)
+
+    def test_top_level_key_not_string(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", ["TopKey2"], "SecondKey1"]
+        }
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(input, output)
+
+    def test_second_level_key_not_string(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", "TopKey1", ["SecondKey2"]]
+        }
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(input, output)
+
+    def test_mapping_not_found(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapB", "TopKey2", "SecondKey1"]
+        }
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(input, output)
+
+    def test_top_level_key_not_found(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", "TopKey3", "SecondKey1"]
+        }
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(input, output)
+
+    def test_second_level_key_not_found(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", "TopKey1", "SecondKey1"]
+        }
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(input, output)
+
+    def test_one_level_find_in_mappings(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "value4"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", "TopKey2", "SecondKey1"]
+        }
+        expected = "value4"
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(expected, output)
+
+    def test_nested_find_in_mappings(self):
+        mappings = {
+            "MapA":{
+                "TopKey1": {
+                   "SecondKey2": "value3"
+               },
+                "TopKey2": {
+                    "SecondKey1": "TopKey1"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", {"Fn::FindInMap": ["MapA", "TopKey2", "SecondKey1"]}, "SecondKey2"]
+        }
+        expected = "value3"
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(expected, output)
+
+    def test_nested_find_in_multiple_mappings(self):
+        mappings = {
+            "MapA":{
+                "ATopKey1": {
+                   "ASecondKey2": "value3"
+               }
+            },
+            "MapB": {
+                "BTopKey1": {
+                    "BSecondKey2": "ATopKey1"
+                }
+            }
+        }
+        input = {
+            "Fn::FindInMap": ["MapA", {"Fn::FindInMap": ["MapB", "BTopKey1", "BSecondKey2"]}, "ASecondKey2"]
+        }
+        expected = "value3"
+        output = self.ref.resolve_parameter_refs(input, mappings)
+
+        self.assertEqual(expected, output)
